unit FdForm;

interface

uses
  Controls, Forms,
  StdCtrls, Mask, GenAlg, Classes;

type
  TForm1 = class(TForm)
    ExecuteButton: TButton;
    ResultLabel: TLabel;
    HaploidGeneticAlgorithm1: THaploidGeneticAlgorithm;
    FeedALabel: TLabel;
    FeedBLabel: TLabel;
    procedure ExecuteButtonClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}
function MyFitnessFunction(var Indiv : Individual) : real;
  var
    minReqProtein : real;
    MinReqFat : real;
    MinReqCarbohydrate : real;
    Protein : real;
    Fat : real;
    carbohydrate : real;
    ReturnValue : real;
  begin                                 {*** MyFitnessFunction ***}
    MinReqProtein := 0.06;              {minimum requirements in kg }
    MinReqFat := 0.024;
    MinReqCarboHydrate := 0.030;
    Protein := ((Indiv.Chrom[1] * 0.020) +
                (Indiv.Chrom[2] * 0.030)) / 10;
    Fat := ((Indiv.Chrom[1] * 0.012) +
            (Indiv.Chrom[2] * 0.006)) / 10;
    Carbohydrate := ((Indiv.Chrom[1] * 0.030) +
                     (Indiv.Chrom[2] * 0.015)) / 10;
    ReturnValue := - (Indiv.Chrom[1] * 0.60 / 10 +
                      Indiv.Chrom[2] * 0.40 / 10);
    if Protein < MinReqProtein then
      ReturnValue := -5;
    if Fat < MinReqFat then
      ReturnValue := -5;
    if carbohydrate < MinReqCarbohydrate then
      ReturnValue := -5;
    Result := ReturnValue;
  end;                                  {*** MyFitnessFunction ***}

function MyMutationFunction(AllelePos : word;
                            OldValue : byte) : Allele;
  begin                                 {*** MyMutationFunction ***}
    Result := Random(101);
  end;                                  {*** MyMutationFunction ***}


procedure TForm1.ExecuteButtonClick(Sender: TObject);
var
  TmpStr : string;
begin
  with HaploidGeneticAlgorithm1 do
    begin
      PopulationSize := 30;
      ChromosomeLength := 2;
      NumGenerations := 50;
      ProbCrossingOver := 0.1;
      ProbMutation := 0.6;
      SelectionProportion := 0.3;
    end;
  HaploidGeneticAlgorithm1.Execute;
  Str(- HaploidGeneticAlgorithm1.
          HighestEverFitness : 10 : 3,
      TmpStr);
  ResultLabel.Caption := 'Lowest Cost: ' + TmpStr;
  Str(HaploidGeneticAlgorithm1.FittestEver[1] / 10 : 2 : 2, TmpStr);
  FeedALabel.Caption := 'Feed A: ' + TmpStr + ' kg';
  Str(HaploidGeneticAlgorithm1.FittestEver[2] / 10 : 2 : 2, TmpStr);
  FeedBLabel.Caption := 'Feed B: ' + TmpStr + ' kg';
end;

procedure TForm1.FormActivate(Sender: TObject);
begin
  HaploidGeneticAlgorithm1.FitnessFunction := MyFitnessFunction;
  HaploidGeneticAlgorithm1.MutateFunction := MyMutationFunction;
end;

end.
