unit GalgExec;
interface

uses
  SysUtils, Classes, Controls, Forms, Dialogs,
  StdCtrls, Mask, ExtCtrls, Buttons,
  GenAlg, DipAlg, Grids, Gauges;

type
  TRationOptimizeForm = class(TForm)
    MinMaxInputPanel: TPanel;
    DMIMinMaskEdit: TMaskEdit;
    DMIMaxMaskEdit: TMaskEdit;
    CPMinMaskEdit: TMaskEdit;
    CPMaxMaskEdit: TMaskEdit;
    MEMinMaskEdit: TMaskEdit;
    ADFMinMaskEdit: TMaskEdit;
    ForageMinMaskEdit: TMaskEdit;
    CaMinMaskEdit: TMaskEdit;
    DMILabel: TLabel;
    MinLabel: TLabel;
    MaxLabel: TLabel;
    CPLabel: TLabel;
    MELabel: TLabel;
    ADFLabel: TLabel;
    ForageLabel: TLabel;
    CaLabel: TLabel;
    PLabel: TLabel;
    CaPLabel: TLabel;
    MEMaxMaskEdit: TMaskEdit;
    ADFMaxMaskEdit: TMaskEdit;
    ForageMaxMaskEdit: TMaskEdit;
    CaMaxMaskEdit: TMaskEdit;
    PMinMaskEdit: TMaskEdit;
    CaPMinMaskEdit: TMaskEdit;
    PMaxMaskEdit: TMaskEdit;
    CaPMaxMaskEdit: TMaskEdit;
    HaploidGeneticAlgorithm: THaploidGeneticAlgorithm;
    ParameterPanel: TPanel;
    PopSizeMaskEdit: TMaskEdit;
    NumGenMaskEdit: TMaskEdit;
    ChromLengthMaskEdit: TMaskEdit;
    PCrossMaskEdit: TMaskEdit;
    PMutationMaskEdit: TMaskEdit;
    popSizeLabel: TLabel;
    ChromLengthLabel: TLabel;
    NumGenLabel: TLabel;
    PCrossLabel: TLabel;
    PMutationLabel: TLabel;
    GenParmsLabel: TLabel;
    ConstraintsLabel: TLabel;
    SelIntMaskEdit: TMaskEdit;
    SelIntLabel: TLabel;
    ResultPanel: TPanel;
    AlfHerLabel: TLabel;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    ExecPanel: TPanel;
    LoadBitBtn: TBitBtn;
    SaveBitBtn: TBitBtn;
    ExecuteBitBtn: TBitBtn;
    FeedCompPanel: TPanel;
    FeedCompStringGrid: TStringGrid;
    FeedCompLabel: TLabel;
    ProgressGauge: TGauge;
    ResultLabel: TLabel;
    FormulaLabel: TLabel;
    NOfRunsMaskEdit: TMaskEdit;
    Label8: TLabel;
    Label9: TLabel;
    OutputFileMaskEdit: TMaskEdit;
    DiploidGeneticAlgorithm: TDiploidGeneticAlgorithm;
    HaploidRadioButton: TRadioButton;
    DiploidRadioButton: TRadioButton;
    procedure ExecuteBitBtnClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure SaveBitBtnClick(Sender: TObject);
    procedure LoadBitBtnClick(Sender: TObject);
    procedure HaploidRadioButtonClick(Sender: TObject);
    procedure DiploidRadioButtonClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    DMI, DMIMin, DMIMax : real;
    CP, CPMin, CPMax : real;
    ME, MEMin, MEMax : real;
    ADF, ADFMin, ADFMax : real;
    Forage, ForageMin, Foragemax : real;
    Ca, CaMin, CaMax : real;
    P, PMin, PMax : real;
    CaP, CaPMin, CaPMax : real;
    Cost : real;
  procedure Execute;
  end;

var
  RationOptimizeForm: TRationOptimizeForm;

implementation

{$R *.DFM}
{$F+}
function DipFitnessFunction(var Indiv : DipAlg.Individual) : real;
  var
    Loop : word;
    Code : integer;
    DMVal : real;                       {Nutrient values of individual }
    EDMVal : real;                      {ingredients }
    CPVal : real;
    MEVal : real;
    ADFVal : real;
    ForageVal : real;
    CaVal : real;
    PVal : real;
    Price : real;

  function LowestOf(Byte1, Byte2 : byte) : byte;
    begin
      if Byte1 < Byte2 then
        Result := Byte1
      else
        Result := Byte2;
    end;

  begin                                 {*** DipFitnessFunction ***}
    with RationOptimizeForm do
      begin
        DMI := 0;                       {initialize ration nutrient }
        CP := 0;                        {composition }
        ME := 0;
        ADF := 0;
        Forage := 0;
        Ca := 0;
        P := 0;
        CaP := 0;
        Cost := 0;
          for Loop := 1 to DiploidGeneticAlgorithm.ChromosomeLength do
            begin
              Indiv.EChrom[Loop] := LowestOf(Indiv.Chrom[1, Loop],
                                             Indiv.Chrom[2, Loop]);
            end;
                                        {values for current ration, based }
                                        {on current 'chromosome ' }
        with FeedCompStringGrid do      {add the values of each individual }
        for Loop := 1 to RowCount do    {feed to the current chromosome }
          if Trim(Cells[0, Loop]) <> '' then      {to calculate ration }
            begin                                 {nutrient composition }
              Val(Cells[1, Loop], DMVal, Code);
              Val(Cells[2, Loop], EDMVal, Code);
              Val(Cells[3, Loop], CPVal, Code);
              Val(Cells[4, Loop], MEVal, Code);
              Val(Cells[5, Loop], ADFVal, Code);
              Val(Cells[6, Loop], ForageVal, Code);
              Val(Cells[7, Loop], CaVal, Code);
              Val(Cells[8, Loop], PVal, Code);
              Val(Cells[10, Loop], Price, Code);
              with Indiv do
                begin
                  DMI := DMI + EChrom[Loop] * DMVal / 100;
                  CP := CP + EChrom[Loop] * CPVal * DMVal / EDMVal / 100;
                  ME := ME + EChrom[Loop] * MEVal * DMVal / EDMVal;
                  ADF := ADF + EChrom[Loop] * ADFVal * DMVal / EDMVal / 100;
                  Forage := Forage + EChrom[Loop] * ForageVal * DMVal /EDMVal;
                  Ca := Ca + EChrom[Loop] * CaVal * DMVal / EDMVal * 453.6 / 100;
                  P := P + EChrom[Loop] * PVal * DMVal / EDMVal * 453.6 / 100;
                end;
              if P > 0.001 then
                CaP := Ca / P;
              Cost := Cost + Price / 2000 * Indiv.EChrom[Loop];
            end;
                                        {Check if ration nutrients are }
                                        {within bound. If not, set a very }
                                        {low value for Fitness }
        if DMI > 0.001 then
          begin
            ADF := ADF / DMI * 100;
            Forage := Forage / DMI * 100;
          end
        else
          begin
            ADF := 0;
            Forage := 0;
          end;
        Result := -Cost;
        if (DMI < DMIMin) or
           (DMI > DMIMax)then
          Result := -999;
        if (CP < CPMin) or
           (CP > CPMax)then
          Result := -999;
        if (CP < CPMin) or
           (CP > CPMax)then
          Result := -999;
        if (ME < MEMin) or
           (ME > MEMax)then
          Result := -999;
        if (ADF < ADFMin) or
           (ADF > ADFMax)then
          Result := -999;
        if (Forage < ForageMin) or
           (Forage > ForageMax)then
          Result := -999;
        if (Ca < CaMin) or
           (Ca > CaMax)then
          Result := -999;
        if (P < PMin) or
           (P > PMax)then
          Result := -999;
        if (CaP < CaPMin) or
           (CaP > CaPMax)then
          Result := -999;
        ProgressGauge.Progress :=       {set value fr progress gauge }
           Round(DiploidGeneticAlgorithm.
                ProgressPercentage);
      end;
  end;                                  {*** DipFitnessFunction ***}

function HapFitnessFunction(var Indiv : GenAlg.Individual) : real;
  var
    Loop : word;
    Code : integer;
    DMVal : real;                       {Nutrient values of individual }
    EDMVal : real;                      {ingredients }
    CPVal : real;
    MEVal : real;
    ADFVal : real;
    ForageVal : real;
    CaVal : real;
    PVal : real;
    Price : real;
  begin                                 {*** HapFitnessFunc ***}
    with RationOptimizeForm do
      begin
        DMI := 0;                       {initialize ration nutrient }
        CP := 0;                        {composition }
        ME := 0;
        ADF := 0;
        Forage := 0;
        Ca := 0;
        P := 0;
        CaP := 0;
        Cost := 0;
                                        {values for current ration, based }
                                        {on current 'chromosome ' }
        with FeedCompStringGrid do      {add the values of each individual }
        for Loop := 1 to RowCount do    {feed to the current chromosome }
          if Trim(Cells[0, Loop]) <> '' then      {to calculate ration }
            begin                                 {nutrient composition }
              Val(Cells[1, Loop], DMVal, Code);
              Val(Cells[2, Loop], EDMVal, Code);
              Val(Cells[3, Loop], CPVal, Code);
              Val(Cells[4, Loop], MEVal, Code);
              Val(Cells[5, Loop], ADFVal, Code);
              Val(Cells[6, Loop], ForageVal, Code);
              Val(Cells[7, Loop], CaVal, Code);
              Val(Cells[8, Loop], PVal, Code);
              Val(Cells[10, Loop], Price, Code);
              DMI := DMI + Indiv.Chrom[Loop] * DMVal / 100;
              CP := CP + Indiv.Chrom[Loop] * CPVal * DMVal / EDMVal / 100;
              ME := ME + Indiv.Chrom[Loop] * MEVal * DMVal / EDMVal;
              ADF := ADF + Indiv.Chrom[Loop] * ADFVal * DMVal / EDMVal / 100;
              Forage := Forage + Indiv.Chrom[Loop] * ForageVal * DMVal /EDMVal;
              Ca := Ca + Indiv.Chrom[Loop] * CaVal * DMVal / EDMVal * 453.6 / 100;
              P := P + Indiv.Chrom[Loop] * PVal * DMVal / EDMVal * 453.6 / 100;
              if P > 0.001 then
                CaP := Ca / P;
              Cost := Cost + Price / 2000 * Indiv.Chrom[Loop];
            end;
                                        {Check if ration nutrients are }
                                        {within bound. If not, set a very }
                                        {low value for Fitness }
        if DMI > 0.001 then
          begin
            ADF := ADF / DMI * 100;
            Forage := Forage / DMI * 100;
          end
        else
          begin
            ADF := 0;
            Forage := 0;
          end;
        Result := -Cost;
        if (DMI < DMIMin) or
           (DMI > DMIMax)then
          Result := -999;
        if (CP < CPMin) or
           (CP > CPMax)then
          Result := -999;
        if (CP < CPMin) or
           (CP > CPMax)then
          Result := -999;
        if (ME < MEMin) or
           (ME > MEMax)then
          Result := -999;
        if (ADF < ADFMin) or
           (ADF > ADFMax)then
          Result := -999;
        if (Forage < ForageMin) or
           (Forage > ForageMax)then
          Result := -999;
        if (Ca < CaMin) or
           (Ca > CaMax)then
          Result := -999;
        if (P < PMin) or
           (P > PMax)then
          Result := -999;
        if (CaP < CaPMin) or
           (CaP > CaPMax)then
          Result := -999;
        ProgressGauge.Progress :=       {set value fr progress gauge }
           Round(HaploidGeneticAlgorithm.
                ProgressPercentage);
      end;
  end;                                  {*** HapFitnessFunc ***}
{$F-}

function MyMutationFunction(AllelePos : word;
                            OldValue : byte) : Allele;
  begin                                 {*** MyMutationFunc ***}
    Result := Random(51);
  end;                                  {*** MyMutationFunc ***}

procedure  TRationOptimizeForm.Execute;
  var
    Code : integer;
    TmpStr : string;
    Loop : byte;
  begin
    Val(DMIMinMaskEdit.Text, DMIMin, Code);       {get values for min and max }
    Val(DMIMaxMaskEdit.Text, DMIMax, Code);       {ingredient bounds }
    Val(CPMinMaskEdit.Text, CPMin, Code);
    Val(CPMaxMaskEdit.Text, CPMax, Code);
    Val(MEMinMaskEdit.Text, MEMin, Code);
    Val(MEMaxMaskEdit.Text, MEMax, Code);
    Val(ADFMinMaskEdit.Text, ADFMin, Code);
    Val(ADFMaxMaskEdit.Text, ADFMax, Code);
    Val(ForageMinMaskEdit.Text, ForageMin, Code);
    Val(ForageMaxMaskEdit.Text, Foragemax, Code);
    Val(CaMinMaskEdit.Text, CaMin, Code);
    Val(CaMaxMaskEdit.Text, CaMax, Code);
    Val(PMinMaskEdit.Text, PMin, Code);
    Val(PMaxMaskEdit.Text, PMax, Code);
    Val(CaPMinMaskEdit.Text, CaPMin, Code);
    Val(CaPMaxMaskEdit.Text, CaPMax, Code);
    if DMIMax < 0.0001  then            {if max bounds <= 0 then set to }
      DMIMax := 99999;                  {very high value }    
    if CPMax < 0.0001 then
      CPMax := 99999;
    if MEMax < 0.0001 then
      MEMax := 99999;
    if ADFMax < 0.0001 then
      ADFMax := 99999;
    if ForageMax < 0.0001 then
      ForageMax := 99999;
    if CaMax < 0.0001 then
      CaMax := 99999;
    if PMax < 0.0001 then
      PMax := 99999;
    if CaPMax < 0.0001 then
      CaPMax := 99999;
    if DMIMin < 0.0001  then            {if min bounds <= 0 then set to }
      DMIMin := -99999;                 {very low value }
    if CPMin < 0.0001 then
      CPMin := -99999;
    if MEMin < 0.0001 then
      MEMin := -99999;
    if ADFMin < 0.0001 then
      ADFMin := -99999;
    if ForageMin < 0.0001 then
      ForageMin := -99999;
    if CaMin < 0.0001 then
      CaMin := -99999;
    if PMin < 0.0001 then
      PMin := -99999;
    if CaPMin < 0.0001 then
      CaPMin := -99999;
    if HaploidRadioButton.Checked then
      begin
        HaploidGeneticAlgorithm.Execute;
        Str(-HaploidGeneticAlgorithm.
              HighestEverFitness : 5 : 2, TmpStr);
      end
    else
      begin
        DiploidGeneticAlgorithm.Execute;
        Str(-DiploidGeneticAlgorithm.
              HighestEverFitness : 5 : 2, TmpStr);
      end;
    ResultLabel.Caption := 'Cost : $' + TmpStr;
    for Loop := 1 to 7 do
      begin
        TmpStr := '';
        if FeedCompStringGrid.
                Cells[0, Loop] <> '' then
          begin
            if HaploidRadioButton.Checked then
              Str(HaploidGeneticAlgorithm.
                   FittestEver[Loop] : 10, TmpStr)
            else
              Str(DiploidGeneticAlgorithm.
                   FittestEver[Loop] : 10, TmpStr);
            TmpStr := Trim(FeedCompStringGrid.
                           Cells[0, Loop]) +
                             ': ' + TmpStr + ' lbs'
          end;
        Case Loop of
          1 : Label1.Caption := TmpStr;
          2 : Label2.Caption := TmpStr;
          3 : Label3.Caption := TmpStr;
          4 : Label4.Caption := TmpStr;
          5 : Label5.Caption := TmpStr;
          6 : Label6.Caption := TmpStr;
          7 : Label7.Caption := TmpStr;
        end;
      end;
  end;

procedure TRationOptimizeForm.ExecuteBitBtnClick(Sender: TObject);
var
  OutFile : TextFile;
  Loop : word;
  TmpStr : string;
  Code : integer;
  TmpWord : word;
  TmpReal : real;
  Cntr : byte;
  NOfRuns : Word;
  FileName : string;
begin
  if DiploidradioButton.Checked then
    begin
      DiploidGeneticAlgorithm.
            FitnessFunction :=
                   DipFitnessFunction;
      DiploidGeneticAlgorithm.
          MutateFunction :=
               MyMutationFunction;
    end
  else
    begin
      HaploidGeneticAlgorithm.              {identify which fitness function is }
              FitnessFunction :=            {to be used }
                   HapFitnessFunction;
      HaploidGeneticAlgorithm.              {identify which mutation function is }
              MutateFunction :=             {to be used }
                   MyMutationFunction;
    end;
  FileName := OutputFileMaskEdit.Text;
  if Trim(FileName) <> '.' then
    begin
      AssignFile(OutFile, FileName);
      Rewrite(OutFile);
      Write(OutFile, 'Cost    ');

      for Loop := 1 to 7 do
        if FeedCompStringGrid.
                  Cells[0, Loop] <> '' then
          Write(OutFile, FeedCompStringGrid.
                                 Cells[0, Loop] + '    ');
      Writeln(OutFile);
    end;
  Cntr := 0;
  for Loop := 1 to 7 do
    if FeedCompStringGrid.Cells[0, Loop] <> '' then
      Inc(Cntr);
  if HaploidRadioButton.Checked then
    HaploidGeneticAlgorithm.
       ChromosomeLength := Cntr
  else
    DiploidGeneticAlgorithm.
       ChromosomeLength := Cntr;
  Str(Cntr, TmpStr);
  ChromLengthMaskEdit.Text := TmpStr;
  if HaploidRadioButton.Checked then
    with HaploidGeneticAlgorithm do     {Get the user entered values for the }
      begin                             {genetic parameters }
        TmpStr := PopSizeMaskEdit.Text;
        Val(TmpStr, TmpWord, Code);
        PopulationSize := TmpWord;
        TmpStr := ChromLengthMaskEdit.Text;
        Val(TmpStr, TmpWord, Code);
        ChromosomeLength := TmpWord;
        TmpStr := NumGenMaskEdit.Text;
        Val(TmpStr, TmpWord, Code);
        NumGenerations := TmpWord;
        TmpStr := PCrossMaskEdit.Text;
        Val(TmpStr, TmpReal, Code);
        ProbCrossingOver := TmpReal;
        TmpStr := PMutationMaskEdit.Text;
        Val(TmpStr, TmpReal, Code);
        ProbMutation := TmpReal;
        TmpStr := SelIntMaskEdit.Text;
        Val(TmpStr, TmpReal, Code);
        SelectionProportion := TmpReal;
      end
  else
    with DiploidGeneticAlgorithm do     {Get the user entered values for the }
      begin                             {genetic parameters }
        TmpStr := PopSizeMaskEdit.Text;
        Val(TmpStr, TmpWord, Code);
        PopulationSize := TmpWord;
        TmpStr := ChromLengthMaskEdit.Text;
        Val(TmpStr, TmpWord, Code);
        ChromosomeLength := TmpWord;
        TmpStr := NumGenMaskEdit.Text;
        Val(TmpStr, TmpWord, Code);
        NumGenerations := TmpWord;
        TmpStr := PCrossMaskEdit.Text;
        Val(TmpStr, TmpReal, Code);
        ProbCrossingOver := TmpReal;
        TmpStr := PMutationMaskEdit.Text;
        Val(TmpStr, TmpReal, Code);
        ProbMutation := TmpReal;
        TmpStr := SelIntMaskEdit.Text;
        Val(TmpStr, TmpReal, Code);
        SelectionProportion := TmpReal;
      end;
  Val(NOfRunsMaskEdit.Text,
      NOfRuns, Code);
  if NOfRuns = 0 then
    NOfRuns := 1;
  for Loop := 1 to NOfRuns do
    begin
      Execute;                          {execute the Algorithm with the }
      if Trim(FileName) <> '.' then
        if haploidRadioButton.Checked then
          begin
            Write(OutFile, -HaploidGeneticAlgorithm.
                  HighestEverFitness : 5 : 2, '       ');
            Write(OutFile, HaploidGeneticAlgorithm.FittestEver[1] : 10, '       ');
            Write(OutFile, HaploidGeneticAlgorithm.FittestEver[2] : 10, '       ');
            Writeln(OutFile, HaploidGeneticAlgorithm.FittestEver[3] : 10, '       ');
          end
        else
          begin
            Write(OutFile, -DiploidGeneticAlgorithm.
                  HighestEverFitness : 5 : 2, '       ');
            Write(OutFile, DiploidGeneticAlgorithm.FittestEver[1] : 10, '       ');
            Write(OutFile, DiploidGeneticAlgorithm.FittestEver[2] : 10, '       ');
            Writeln(OutFile, DiploidGeneticAlgorithm.FittestEver[3] : 10, '       ');
          end;
      Repaint;
    end;
  if Trim(FileName) <> '.' then
    CloseFile(OutFile);
end;

procedure TRationOptimizeForm.FormCreate(Sender: TObject);
var
  TmpStr : string;
begin
  with DiploidGeneticAlgorithm do       {initialize the genetic parameters }
    begin                               {and also put them in the Edit Masks }
      PopulationSize := 500;
      ChromosomeLength := 3;
      NumGenerations := 10;
      ProbCrossingOver := 0.5;
      ProbMutation := 0.5;
      SelectionProportion := 0.05;
    end;
  with HaploidGeneticAlgorithm do       {initialize the genetic parameters }
    begin                               {and also put them in the Edit Masks }
      PopulationSize := 500;
      ChromosomeLength := 3;
      NumGenerations := 10;
      ProbCrossingOver := 0.5;
      ProbMutation := 0.5;
      SelectionProportion := 0.05;
    end;
  with HaploidGeneticAlgorithm do
    begin
      Str(PopulationSize, TmpStr);
      PopSizeMaskEdit.Text := TmpStr;
      Str(ChromosomeLength, TmpStr);
      ChromLengthMaskEdit.Text := TmpStr;
      Str(NumGenerations, TmpStr);
      NumGenMaskEdit.Text := TmpStr;
      Str(ProbCrossingOver : 2 : 2, TmpStr);
      PCrossMaskEdit.Text := TmpStr;
      Str(ProbMutation : 2 : 2, TmpStr);
      PMutationMaskEdit.Text := TmpStr;
      Str(SelectionProportion : 2 : 2, TmpStr);
      SelIntMaskEdit.Text := TmpStr;
    end;
  Cursor := crHourGlass;
  with FeedCompStringGrid do            {put codes in row for nutrients to }
    begin                               {be used }
      Cells[1, 0] := 'DM(%)';
      Cells[2, 0] := 'EDM(%)';
      Cells[3, 0] := 'CP (%EDM)';
      Cells[4, 0] := 'ME (MCal/lb EDM)';
      Cells[5, 0] := 'ADF (%EDM)';
      Cells[6, 0] := 'Forage (0/1)';
      Cells[7, 0] := 'Ca %EDM)';
      Cells[8, 0] := 'P (%EDM)';
      Cells[9, 0] := 'Ca:P';
      Cells[10, 0] := 'Price/ton';
    end;
  Cursor := crArrow;
end;


procedure TRationOptimizeForm.SaveBitBtnClick(Sender: TObject);
var
  FVar : TextFile;
  RowNr, ColNr : word;
  CurDir : string;
begin
  CurDir := paramStr(0);
  while CurDir[Length(CurDir)] <> '\' do
    Delete(CurDir, Length(CurDir), 1);
  AssignFile(FVar, CurDir +
                 'OptiFeed.dat');       {save input data to file }
  Rewrite(FVar);
  Writeln(FVar, DMIMinMaskEdit.Text);
  Writeln(FVar, DMIMaxMaskEdit.Text);
  Writeln(FVar, CPMinMaskEdit.Text);
  Writeln(FVar, CPMaxMaskEdit.Text);
  Writeln(FVar, MEMinMaskEdit.Text);
  Writeln(FVar, MEMaxMaskEdit.Text);
  Writeln(FVar, ADFMinMaskEdit.Text);
  Writeln(FVar, ADFMaxMaskEdit.Text);
  Writeln(FVar, ForageMinMaskEdit.Text);
  Writeln(FVar, ForageMaxMaskEdit.Text);
  Writeln(FVar, CaMinMaskEdit.Text);
  Writeln(FVar, CaMaxMaskEdit.Text);
  Writeln(FVar, PMinMaskEdit.Text);
  Writeln(FVar, PMaxMaskEdit.Text);
  Writeln(FVar, CaPMinMaskEdit.Text);
  Writeln(FVar, CaPMaxMaskEdit.Text);
  with FeedCompStringGrid do
    for RowNr := 1 to RowCount do
      if Trim(Cells[0, RowNr]) <> '' then
        for ColNr := 0 to 10 do
          Writeln(FVar, Cells[ColNr, RowNr]);
  CloseFile(FVar);
end;

procedure TRationOptimizeForm.LoadBitBtnClick(Sender: TObject);
var
  FVar : TextFile;
  ColNr, RowNr : word;
  TmpStr : String;
  CurDir : string;
begin
  CurDir := paramStr(0);
  while CurDir[Length(CurDir)] <> '\' do
    Delete(CurDir, Length(CurDir), 1);
  AssignFile(FVar, CurDir +
                 'OptiFeed.dat');       {read input data to file }
  Reset(FVar);
  Readln(FVar, TmpStr);
  DMIMinMaskEdit.Text := TmpStr;
  Readln(FVar, TmpStr);
  DMIMaxMaskEdit.Text := TmpStr;
  Readln(FVar, TmpStr);
  CPMinMaskEdit.Text := TmpStr;
  Readln(FVar, TmpStr);
  CPMaxMaskEdit.Text := TmpStr;
  Readln(FVar, TmpStr);
  MEMinMaskEdit.Text := TmpStr;
  Readln(FVar, TmpStr);
  MEMaxMaskEdit.Text := TmpStr;
  Readln(FVar, TmpStr);
  ADFMinMaskEdit.Text := TmpStr;
  Readln(FVar, TmpStr);
  ADFMaxMaskEdit.Text := TmpStr;
  Readln(FVar, TmpStr);
  ForageMinMaskEdit.Text := TmpStr;
  Readln(FVar, TmpStr);
  ForageMaxMaskEdit.Text := TmpStr;
  Readln(FVar, TmpStr);
  CaMinMaskEdit.Text := TmpStr;
  Readln(FVar, TmpStr);
  CaMaxMaskEdit.Text := TmpStr;
  Readln(FVar, TmpStr);
  PMinMaskEdit.Text := TmpStr;
  Readln(FVar, TmpStr);
  PMaxMaskEdit.Text := TmpStr;
  Readln(FVar, TmpStr);
  CaPMinMaskEdit.Text := TmpStr;
  Readln(FVar, TmpStr);
  CaPMaxMaskEdit.Text := TmpStr;
  RowNr := 0;
  with FeedCompStringGrid do
    while not Eof(FVar) do
      begin
        Inc(RowNr);
        for ColNr := 0 to 10 do
          begin
            Readln(FVar, TmpStr);
            Cells[ColNr, RowNr] := TmpStr;
          end;
      end;
  CloseFile(FVar);
  Label1.Caption := '';
  Label2.Caption := '';
  Label3.Caption := '';
  Label4.Caption := '';
  Label5.Caption := '';
  Label6.Caption := '';
  Label7.Caption := '';
end;


procedure TRationOptimizeForm.HaploidRadioButtonClick(Sender: TObject);
var
  TmpStr : string;
begin
  With HaploidGeneticAlgorithm do
    begin
      Str(PopulationSize, TmpStr);
      PopSizeMaskEdit.Text := TmpStr;
      Str(ChromosomeLength, TmpStr);
      ChromLengthMaskEdit.Text := TmpStr;
      Str(NumGenerations, TmpStr);
      NumGenMaskEdit.Text := TmpStr;
      Str(ProbCrossingOver : 2 : 2, TmpStr);
      PCrossMaskEdit.Text := TmpStr;
      Str(ProbMutation : 2 : 2, TmpStr);
      PMutationMaskEdit.Text := TmpStr;
      Str(SelectionProportion : 2 : 2, TmpStr);
      SelIntMaskEdit.Text := TmpStr;
    end;
end;

procedure TRationOptimizeForm.DiploidRadioButtonClick(Sender: TObject);
var
  TmpStr : string;
begin
  with DiploidGeneticAlgorithm do
    begin
      Str(PopulationSize, TmpStr);
      PopSizeMaskEdit.Text := TmpStr;
      Str(ChromosomeLength, TmpStr);
      ChromLengthMaskEdit.Text := TmpStr;
      Str(NumGenerations, TmpStr);
      NumGenMaskEdit.Text := TmpStr;
      Str(ProbCrossingOver : 2 : 2, TmpStr);
      PCrossMaskEdit.Text := TmpStr;
      Str(ProbMutation : 2 : 2, TmpStr);
      PMutationMaskEdit.Text := TmpStr;
      Str(SelectionProportion : 2 : 2, TmpStr);
      SelIntMaskEdit.Text := TmpStr;
    end;
end;

end.
