unit tsp;
interface

uses
  Controls, Forms,
  GenAlg, StdCtrls, Classes;
type
  TForm1 = class(TForm)
    HaploidGeneticAlgorithm1: THaploidGeneticAlgorithm;
    RunButton: TButton;
    Memo1: TMemo;
    procedure FormActivate(Sender: TObject);
    procedure RunButtonClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;
implementation

{$R *.DFM}
type
  City = (Amsterdam, Rotterdam, TheHague,
          Utrecht, Nijmegen);
const
  CityNames : array[1..5] of string =
         ('Amsterdam', 'Rotterdam', 'The Hague',
          'Utrecht', 'Nijmegen');
var
  DistanceTable : array[1..5,1..5] of word;



{$F+}
function ChangeGene(AllelePos : word;
                    OldValue : byte) : Allele;
  begin
    Result := 1 + Random(5);
  end;

function TotalDistance(var Indiv : Individual) : real;
  var
    Loop : byte;
    Loop1 : byte;
    Value : byte;
    Unique : boolean;
    Distance : word;
  begin
    Unique := TRUE;                     {first check if all city numbers are }
    for Loop := 1 to 5 do               {unique. If not, then the individual is}
      begin                             {unfit }
        Value := Indiv.Chrom[Loop];
        for Loop1 := 1 to 5 do
          if (Loop <> Loop1) and
             (Indiv.Chrom[Loop1] = Value) then
            Unique := FALSE;
      end;
    if not Unique then                  {if one city occurs twice or more then }
      Result := -9999                   {set fitness value to very low }
    else                                {otherwise, calculate the negative of }
      begin                             {the total distance }
        Distance := 0;                  {initialize distance }
        for Loop := 1 to 4 do           {add distances between cities }
          Distance := Distance +
                         DistanceTable[Indiv.Chrom[Loop],
                                    Indiv.Chrom[Loop + 1]];
        Distance := Distance +          {for the trip back to start city }
                         DistanceTable[Indiv.Chrom[5],
                                       Indiv.Chrom[1]];
        Result := -Distance;
      end;
  end;
{$F-}

procedure TForm1.FormActivate(Sender: TObject);
  procedure SetDistanceTable(Row, Col : City; Value : word);
    var
      R, C : word;
    begin
      R := word(Row) + 1;
      C := word(Col) + 1;
      DistanceTable[R, C] := Value;
      DistanceTable[C, R] := Value;
    end;
begin
  SetDistanceTable(Amsterdam, Amsterdam, 0);
  SetDistanceTable(Amsterdam, Rotterdam, 73);
  SetDistanceTable(Amsterdam, TheHague, 84);
  SetDistanceTable(Amsterdam, Utrecht, 37);
  SetDistanceTable(Amsterdam, Nijmegen, 122);
  SetDistancetable(Rotterdam, Rotterdam, 0);
  SetDistancetable(Rotterdam, TheHague, 21);
  SetDistancetable(Rotterdam, Utrecht, 57);
  SetDistancetable(Rotterdam, Nijmegen, 114);
  SetDistanceTable(TheHague, TheHague, 0);
  SetDistanceTable(TheHague, Utrecht, 62);
  SetDistanceTable(TheHague, Nijmegen, 135);
  SetDistancetable(Utrecht, Utrecht, 0);
  SetDistanceTable(Utrecht, Nijmegen, 85);
end;

procedure TForm1.RunButtonClick(Sender: TObject);
var
  TmpStr : string;
  Index : byte;
begin
  HaploidGeneticAlgorithm1.
       FitnessFunction := TotalDistance;
  HaploidGeneticAlgorithm1.
       MutateFunction := ChangeGene;
  HaploidGeneticAlgorithm1.
       ChromosomeLength := 5;
  HaploidGeneticAlgorithm1.Execute;
  Memo1.Clear;
  with HaploidGeneticAlgorithm1 do
    begin
      Index := FittestEver[1];
      Memo1.Lines.Add(Citynames[Index]);
      Index := FittestEver[2];
      Memo1.Lines.Add(Citynames[Index]);
      Index := FittestEver[3];
      Memo1.Lines.Add(Citynames[Index]);
      Index := FittestEver[4];
      Memo1.Lines.Add(Citynames[Index]);
      Index := FittestEver[5];
      Memo1.Lines.Add(Citynames[Index]);
      Str(-HighestEverFitness : 2 : 0, TmpStr);
      TmpStr := 'Total Distance ' + TmpStr;
      Memo1.Lines.Add(TmpStr);
    end;
end;

end.
